#include "SchedulerInitStack.h"

int deepStack ;

void  dumpByteLine( int address, int size, int addressDisplay )
{
    if( addressDisplay ) {
        dump2Byte( address ) ;
    }
    for( int i = 0 ; i < size ; i ++ ) {
        space( 1 ) ;
        dumpByte( *( ( char* )( address ) ) ) ;
        address ++ ;
    }
}
void  dumpByteLineChar( int address, int size )
{
    for( int i = 0 ; i < size ; i ++ ) {
        unsigned char data = *( ( char* )( address ) ) ;
        if( isPrintableChar( data ) ) {
            Serial.print( ( char )data ) ;
        }
        else {
            Serial.print( ' ') ;
        }
        address ++ ;
    }
}
char isPrintableChar( unsigned char data )
{
    if( ( ' ' <= data ) && ( data <= 0x7f ) ) {
        return 1 ;
    }
    else if( ( 0xa1 <= data ) && ( data <= 0xdf ) ) {
        return 1 ;
    }
    return 0 ;
}
void  dumpRomByteLine( int address, int size, int addressDisplay )
{
    if( addressDisplay ) {
        dump2Byte( address ) ;
    }
    for( int i = 0 ; i < size ; i ++ ) {
        space( 1 ) ;
        dumpByte( pgm_read_byte( address ) ) ;
        address ++ ;
    }
}
void  dumpRomByteLineChar( int address, int size )
{
    for( int i = 0 ; i < size ; i ++ ) {
        unsigned char data = pgm_read_byte( address ) ;
        if( isPrintableChar( data ) ) {
            Serial.print( ( char )data ) ;
        }
        else {
            Serial.print( ' ') ;
        }
        address ++ ;
    }
}
void crlf( void )
{
    cr() ;
    Serial.write( '\n' ) ;
}
void cr( void )
{
    Serial.write( '\r' ) ;
}
void space( int num )
{
    for( int i = 0 ; i < num ; i ++ ) {
        Serial.print( ' ') ;
    }
}
void dump2Byte( int data )
{
    dumpByte( data >> 8 ) ;
    dumpByte( data ) ;
}
void dumpByte( int data )
{
    a16( data >> 4 ) ;
    a16( data ) ;
}
void a16( char data )
{
    data &= 0xf ;
    Serial.print( data , HEX ) ;
    deepStack = GetSp() ;
}
// http://elm-chan.org/fsw/strf/xprintf_j.html
int xatoi (         /* 0:Failed, 1:Successful */
    char **str,     /* Pointer to pointer to the string */
    long *res       /* Pointer to the valiable to store the value */
)
{
    unsigned long val;
    unsigned char c, r, s = 0;


    *res = 0;

    while ((c = **str) == ' ') (*str)++;    /* Skip leading spaces */

    if (c == '-') {     /* negative? */
        s = 1;
        c = *(++(*str));
    }

    if (c == '0') {
        c = *(++(*str));
        switch (c) {
        case 'x':       /* hexdecimal */
            r = 16; c = *(++(*str));
            break;
        case 'b':       /* binary */
            r = 2; c = *(++(*str));
            break;
        default:
            if (c <= ' ') return 1; /* single zero */
            if (c < '0' || c > '9') return 0;   /* invalid char */
            r = 8;      /* octal */
        }
    } else {
        if (c < '0' || c > '9') return 0;   /* EOL or invalid char */
        r = 10;         /* decimal */
    }

    val = 0;
    while (c > ' ') {
        if (c >= 'a') c -= 0x20;
        c -= '0';
        if (c >= 17) {
            c -= 7;
            if (c <= 9) return 0;   /* invalid char */
        }
        if (c >= r) return 0;       /* invalid char for current radix */
        val = val * r + c;
        c = *(++(*str));
    }
    if (s) val = 0 - val;           /* apply sign if needed */

    *res = val;
    return 1;
}
int getDeepStack( void )
{
    return deepStack ;
}
uint8_t* short2byte( uint16_t data )
{
    static uint8_t buf[ 2 ] ;
    buf[ 0 ] = ( uint8_t )data ;
    buf[ 1 ] = ( uint8_t )( data >> 8 ) ;
    return buf ;
}
uint16_t byte2short( uint8_t data[] )
{
    uint16_t ans ;
    ans = data[ 0 ] ;
    ans &= 0xff ;
    ans |= ( ( ( uint16_t )( data[ 1 ] ) ) << 8 )  ;
    return ans ;
}
uint32_t byte2long( uint8_t data[] )
{
    uint32_t ans ;
    ans = ( uint32_t )( byte2short( data ) ) ;
    ans &= 0xffff ;
    ans |= ( ( ( uint32_t )( byte2short( &( data[ 2 ] ) ) ) ) << 16 ) ;
    return ans ;
}
uint8_t* long2byte( uint32_t data )
{
    static uint8_t buf[ 4 ] ;
    buf[ 0 ] = ( uint8_t )data ;
    buf[ 1 ] = ( uint8_t )( data >> 8 ) ;
    buf[ 2 ] = ( uint8_t )( data >> 16 ) ;
    buf[ 3 ] = ( uint8_t )( data >> 24 ) ;
    return buf ;
}

