#include <Arduino.h>
#include <avr/pgmspace.h> // 必要です
#include "mydefine.h"
#include "memdebug.h"
#include "monitor.h"
#include "etcstr.h"
#include "SchedulerInitStack.h"

static void BlinkBlueLedForConfirmLive( void ) ;
static char GetChar( void ) ;
static char* GetCommandLine( void ) ;
static void DumpRom( void );
static void DumpRam( void );
static char kbhit( void );
static void xputc( char a );


#define OnBoardLed  13
#define BS 8
#define VERSTR "Ver 1.0"

static void HowToUse( void )
{
    printFln(VERSTR);
    printFln("d dump ram");
    printFln("e dump rom");
    printFln("v version");
}

void Monitor( void )
{
    uint8_t length ;
    int i ;
    unsigned short key ;
    char* Buffer ;
    char* uid ;
    long value ;

    pinMode(OnBoardLed, OUTPUT);

    while(1) {
        crlf() ;
        printF( "CA >" ) ;
        Buffer = GetCommandLine() ;
        if( Buffer == 0 ) {
            continue ;
        }
        crlf() ;
        i = toupper( (int)*Buffer ) ;
        Buffer ++ ;
        switch( i ) {
        case 'D' :
            DumpRam() ;
            printF("getFreeMemory()=" ) ;Serial.println((int)getFreeMemory(), DEC);
            crlf() ;
            break ;
        case 'E' :
            DumpRom() ;
            crlf() ;
            break ;
        case 'V' :
            printFln(VERSTR);
            break ;
        default :
            HowToUse() ;
            break ;
        }
    }
}
#define MAX_COMMAND 20
/******************************************************************************
機能要約 1行取得
機能
引き数
戻り値
注意
******************************************************************************/
static char* GetCommandLine( void )
{
    char Data ;
    short i = 0 ;
    static char CommandBuffer[ MAX_COMMAND + 1 ] ;
    //頭についたコントロールコード等を削除する.
    for(;;) {
        Data = GetChar() ;
        if( Data > ' ' ) {
            xputc( Data ) ;
            break ;
        }
    }
    i = 0 ;
    CommandBuffer[ i ] = ( char )Data ;

    for( i = 1 ; i < MAX_COMMAND ; ) {
        Data = GetChar() ;
        //BS の場合.
        if( Data == BS ) {
            if( i > 0 ) {
                xputc( BS ) ;
                xputc( ' ' ) ;
                xputc( BS ) ;
                i -- ;
            }
            continue ;
        }
        // CR とか LF の場合.
        if( Data < ' ' ) {
            CommandBuffer[ i ] = 0 ;
            xputc( Data ) ;
            strlwr( CommandBuffer ) ;
            return( CommandBuffer ) ;
        }

        //通常のキャラクタの場合.
        CommandBuffer[ i ] = Data ;
        xputc( Data ) ;
        i ++ ;
    }
    return( 0 ) ;
}
static char GetChar( void )//1文字取得と共に、動作チェック用LEDの点滅や簡単な処理を行う。
{
    while(1) {
        if( kbhit() ) {
            return Serial.read();
        }
        Scheduler.delay(10);
    }
}
static void xputc( char a )
{
    Serial.print( a ) ;
}
static char kbhit( void )
{
    BlinkBlueLedForConfirmLive() ;//動作チェック用
    if( Serial.available() != 0 ) {
        return 1 ;
    }
    return 0 ;
}


static void BlinkBlueLedForConfirmLive( void )
{
#if 1
    const long mask = 0x1f0ul ; /* 0000 0000 0000 0000 0011 1000 0000 0000 */
    long nowTime = millis() ;
    nowTime &= mask ;
    if( nowTime == 0 ) {
        digitalWrite(OnBoardLed, LOW);
    }
    else {
        digitalWrite(OnBoardLed, HIGH);
    }
#endif
}

#define DUMP_LINE_BYTE 16
static void DumpRam( void )
{
    const int endAddress = (int)RAMEND ;
    for( int address = 0 ; address < endAddress ; address += DUMP_LINE_BYTE ) {
        dumpByteLine( address, DUMP_LINE_BYTE, 1 ) ;
        space(1);
        dumpByteLineChar(address, DUMP_LINE_BYTE ) ;
        crlf() ;
        Scheduler.delay( 10 ) ;
        if( kbhit() ) {
            break ;
        }
    }
}
static void DumpRom( void )
{
    const int endAddress = 0x7ff0 ;
    for( int address = 0 ; address < endAddress ; address += DUMP_LINE_BYTE ) {
        dumpRomByteLine( address, DUMP_LINE_BYTE, 1 ) ;
        space(1);
        dumpRomByteLineChar(address, DUMP_LINE_BYTE ) ;
        crlf() ;
        Scheduler.delay( 10 ) ;
        if( kbhit() ) {
            break ;
        }
    }
}
